from typing import Final

LONG_PASSWORD: Final[int]
FOUND_ROWS: Final[int]
LONG_FLAG: Final[int]
CONNECT_WITH_DB: Final[int]
NO_SCHEMA: Final[int]
COMPRESS: Final[int]
ODBC: Final[int]
LOCAL_FILES: Final[int]
IGNORE_SPACE: Final[int]
PROTOCOL_41: Final[int]
INTERACTIVE: Final[int]
SSL: Final[int]
IGNORE_SIGPIPE: Final[int]
TRANSACTIONS: Final[int]
SECURE_CONNECTION: Final[int]
MULTI_STATEMENTS: Final[int]
MULTI_RESULTS: Final[int]
PS_MULTI_RESULTS: Final[int]
PLUGIN_AUTH: Final[int]
CONNECT_ATTRS: Final[int]
PLUGIN_AUTH_LENENC_CLIENT_DATA: Final[int]
CAPABILITIES: Final[int]
HANDLE_EXPIRED_PASSWORDS: Final[int]
SESSION_TRACK: Final[int]
DEPRECATE_EOF: Final[int]



from _typeshed import Incomplete
from typing import Final

DEBUG: Final[bool]
NULL_COLUMN: Final[int]
UNSIGNED_CHAR_COLUMN: Final[int]
UNSIGNED_SHORT_COLUMN: Final[int]
UNSIGNED_INT24_COLUMN: Final[int]
UNSIGNED_INT64_COLUMN: Final[int]

def dump_packet(data) -> None: ...

class MysqlPacket:
    __slots__ = ("_position", "_data")
    def __init__(self, data, encoding) -> None: ...
    def get_all_data(self): ...
    def read(self, size): ...
    def read_all(self): ...
    def advance(self, length: int) -> None: ...
    def rewind(self, position: int = 0) -> None: ...
    def get_bytes(self, position: int, length: int = 1): ...
    def read_uint8(self): ...
    def read_uint16(self): ...
    def read_uint24(self): ...
    def read_uint32(self): ...
    def read_uint64(self): ...
    def read_string(self): ...
    def read_length_encoded_integer(self) -> Incomplete | None: ...
    def read_length_coded_string(self): ...
    def read_struct(self, fmt: str): ...
    def is_ok_packet(self) -> bool: ...
    def is_eof_packet(self) -> bool: ...
    def is_auth_switch_request(self) -> bool: ...
    def is_extra_auth_data(self) -> bool: ...
    def is_resultset_packet(self) -> bool: ...
    def is_load_local_packet(self) -> bool: ...
    def is_error_packet(self) -> bool: ...
    def check_error(self) -> None: ...
    def raise_for_error(self) -> None: ...
    def dump(self) -> None: ...

class FieldDescriptorPacket(MysqlPacket):
    def __init__(self, data, encoding) -> None: ...
    def description(self): ...
    def get_column_length(self): ...

class OKPacketWrapper:
    def __init__(self, from_packet) -> None: ...
    # TODO: add attrs from `from_packet`
    def __getattr__(self, key: str): ...

class EOFPacketWrapper:
    def __init__(self, from_packet) -> None: ...
    # TODO: add attrs from `from_packet`
    def __getattr__(self, key: str): ...

class LoadLocalPacketWrapper:
    def __init__(self, from_packet) -> None: ...
    # TODO: add attrs from `from_packet`
    def __getattr__(self, key: str): ...



import configparser
import sys
from collections.abc import Mapping, Sequence

class Parser(configparser.RawConfigParser):
    # __init__ signature was taken from RawConfigParser, but with no allow_no_value argument and all arguments are keyword-only
    if sys.version_info >= (3, 13):
        def __init__(
            self,
            *,
            defaults: Mapping[str, str] | None = None,
            dict_type: type[Mapping[str, str]] = ...,
            delimiters: Sequence[str] = ("=", ":"),
            comment_prefixes: Sequence[str] = ("#", ";"),
            inline_comment_prefixes: Sequence[str] | None = None,
            strict: bool = True,
            empty_lines_in_values: bool = True,
            default_section: str = "DEFAULT",
            interpolation: configparser.Interpolation | None = ...,
            converters: configparser._ConvertersMap = ...,
            allow_unnamed_section: bool = False,
        ) -> None: ...
    else:
        def __init__(
            self,
            *,
            defaults: Mapping[str, str] | None = None,
            dict_type: type[Mapping[str, str]] = ...,
            delimiters: Sequence[str] = ("=", ":"),
            comment_prefixes: Sequence[str] = ("#", ";"),
            inline_comment_prefixes: Sequence[str] | None = None,
            strict: bool = True,
            empty_lines_in_values: bool = True,
            default_section: str = "DEFAULT",
            interpolation: configparser.Interpolation | None = ...,
            converters: configparser._ConvertersMap = ...,
        ) -> None: ...

    def optionxform(self, key: str) -> str: ...
    def get(self, section: configparser._SectionName, option: str) -> str: ...  # type: ignore[override]



from _typeshed import FileDescriptorOrPath, Incomplete, Unused
from collections.abc import Callable, Mapping
from socket import _Address, socket as _socket
from ssl import SSLContext, _PasswordType
from typing import Any, AnyStr, Generic, overload
from typing_extensions import Self, TypeVar, deprecated

from .charset import charset_by_id as charset_by_id, charset_by_name as charset_by_name
from .constants import CLIENT as CLIENT, COMMAND as COMMAND, FIELD_TYPE as FIELD_TYPE, SERVER_STATUS as SERVER_STATUS
from .cursors import Cursor
from .err import (
    DatabaseError,
    DataError,
    Error,
    IntegrityError,
    InterfaceError,
    InternalError,
    NotSupportedError,
    OperationalError,
    ProgrammingError,
    Warning,
)

_C = TypeVar("_C", bound=Cursor, default=Cursor)
_C2 = TypeVar("_C2", bound=Cursor)

SSL_ENABLED: bool
DEFAULT_USER: str | None
DEBUG: bool
DEFAULT_CHARSET: str
TEXT_TYPES: set[int]
MAX_PACKET_LEN: int

def dump_packet(data): ...
def _lenenc_int(i: int) -> bytes: ...

class Connection(Generic[_C]):
    ssl: bool
    host: str
    port: int
    user: str | bytes | None
    password: bytes
    db: str | bytes | None
    unix_socket: _Address | None
    charset: str
    collation: str | None
    bind_address: str | None
    use_unicode: bool
    client_flag: int
    cursorclass: type[_C]
    connect_timeout: float | None
    host_info: str
    sql_mode: str | None
    init_command: str | None
    max_allowed_packet: int
    server_public_key: bytes | None
    encoding: str
    autocommit_mode: bool | None
    encoders: dict[type[Any], Callable[[Any], str]]  # argument type depends on the key
    decoders: dict[int, Callable[[str], Any]]  # return type depends on the key

    @overload
    def __init__(
        self,
        *,
        user: str | bytes | None = None,
        password: str | bytes = "",
        host: str | None = None,
        database: str | bytes | None = None,
        unix_socket: _Address | None = None,
        port: int = 0,
        charset: str = "",
        collation: str | None = None,
        sql_mode: str | None = None,
        read_default_file: str | None = None,
        conv: dict[int | type[Any], Callable[[Any], str] | Callable[[str], Any]] | None = None,
        use_unicode: bool = True,
        client_flag: int = 0,
        cursorclass: type[_C] = ...,
        init_command: str | None = None,
        connect_timeout: float = 10,
        read_default_group: str | None = None,
        autocommit: bool | None = False,
        local_infile: bool = False,
        max_allowed_packet: int = 16_777_216,
        defer_connect: bool = False,
        auth_plugin_map: dict[str, Callable[[Connection[Any]], Any]] | None = None,
        read_timeout: float | None = None,
        write_timeout: float | None = None,
        bind_address: str | None = None,
        binary_prefix: bool = False,
        program_name: str | None = None,
        server_public_key: bytes | None = None,
        ssl: dict[str, Incomplete] | SSLContext | None = None,
        ssl_ca: str | None = None,
        ssl_cert: str | None = None,
        ssl_disabled: bool | None = None,
        ssl_key: str | None = None,
        ssl_key_password: _PasswordType | None = None,
        ssl_verify_cert: bool | None = None,
        ssl_verify_identity: bool | None = None,
        compress: Unused = None,
        named_pipe: Unused = None,
        # different between overloads:
        passwd: None = None,  # deprecated
        db: None = None,  # deprecated
    ) -> None: ...
    @overload
    @deprecated("'passwd' and 'db' arguments are deprecated. Use 'password' and 'database' instead.")
    def __init__(
        self,
        *,
        user: str | bytes | None = None,
        password: str | bytes = "",
        host: str | None = None,
        database: str | bytes | None = None,
        unix_socket: _Address | None = None,
        port: int = 0,
        charset: str = "",
        collation: str | None = None,
        sql_mode: str | None = None,
        read_default_file: str | None = None,
        conv: dict[int | type[Any], Callable[[Any], str] | Callable[[str], Any]] | None = None,
        use_unicode: bool = True,
        client_flag: int = 0,
        cursorclass: type[_C] = ...,
        init_command: str | None = None,
        connect_timeout: float = 10,
        read_default_group: str | None = None,
        autocommit: bool | None = False,
        local_infile: bool = False,
        max_allowed_packet: int = 16_777_216,
        defer_connect: bool = False,
        auth_plugin_map: dict[str, Callable[[Connection[Any]], Any]] | None = None,
        read_timeout: float | None = None,
        write_timeout: float | None = None,
        bind_address: str | None = None,
        binary_prefix: bool = False,
        program_name: str | None = None,
        server_public_key: bytes | None = None,
        ssl: dict[str, Incomplete] | SSLContext | None = None,
        ssl_ca: str | None = None,
        ssl_cert: str | None = None,
        ssl_disabled: bool | None = None,
        ssl_key: str | None = None,
        ssl_key_password: _PasswordType | None = None,
        ssl_verify_cert: bool | None = None,
        ssl_verify_identity: bool | None = None,
        compress: Unused = None,
        named_pipe: Unused = None,
        # different between overloads:
        passwd: str | bytes | None = None,  # deprecated
        db: str | bytes | None = None,  # deprecated
    ) -> None: ...
    def close(self) -> None: ...
    @property
    def open(self) -> bool: ...
    def __del__(self) -> None: ...
    def autocommit(self, value) -> None: ...
    def get_autocommit(self) -> bool: ...
    def commit(self) -> None: ...
    def begin(self) -> None: ...
    def rollback(self) -> None: ...
    def select_db(self, db) -> None: ...
    def escape(self, obj, mapping: Mapping[str, Incomplete] | None = None): ...
    def literal(self, obj): ...
    def escape_string(self, s: AnyStr) -> AnyStr: ...
    @overload
    def cursor(self, cursor: None = None) -> _C: ...
    @overload
    def cursor(self, cursor: type[_C2]) -> _C2: ...
    def query(self, sql, unbuffered: bool = False) -> int: ...
    def next_result(self, unbuffered: bool = False) -> int: ...
    def affected_rows(self): ...
    def kill(self, thread_id): ...
    def ping(self, reconnect: bool = True) -> None: ...
    @deprecated("Method is deprecated. Use set_character_set() instead.")
    def set_charset(self, charset: str) -> None: ...
    def set_character_set(self, charset: str, collation: str | None = None) -> None: ...
    def connect(self, sock: _socket | None = None) -> None: ...
    def write_packet(self, payload) -> None: ...
    def _read_packet(self, packet_type=...): ...
    def insert_id(self): ...
    def thread_id(self): ...
    def character_set_name(self): ...
    def get_host_info(self) -> str: ...
    def get_proto_info(self): ...
    def get_server_info(self): ...
    def show_warnings(self): ...
    def __enter__(self) -> Self: ...
    def __exit__(self, *exc_info: object) -> None: ...
    Warning: type[Warning]
    Error: type[Error]
    InterfaceError: type[InterfaceError]
    DatabaseError: type[DatabaseError]
    DataError: type[DataError]
    OperationalError: type[OperationalError]
    IntegrityError: type[IntegrityError]
    InternalError: type[InternalError]
    ProgrammingError: type[ProgrammingError]
    NotSupportedError: type[NotSupportedError]

class MySQLResult:
    connection: Connection[Any] | None
    affected_rows: int | None
    insert_id: int | None
    server_status: int | None
    warning_count: int
    message: str | None
    field_count: int
    description: Incomplete
    rows: Incomplete
    has_next: bool | None
    unbuffered_active: bool
    def __init__(self, connection: Connection[Any]) -> None: ...
    def __del__(self) -> None: ...
    first_packet: Incomplete
    def read(self) -> None: ...
    def init_unbuffered_query(self) -> None: ...

class LoadLocalFile:
    filename: FileDescriptorOrPath
    connection: Connection[Any]
    def __init__(self, filename: FileDescriptorOrPath, connection: Connection[Any]) -> None: ...
    def send_data(self) -> None: ...



from typing import Final

CR_ERROR_FIRST: Final = 2000
CR_UNKNOWN_ERROR: Final = 2000
CR_SOCKET_CREATE_ERROR: Final = 2001
CR_CONNECTION_ERROR: Final = 2002
CR_CONN_HOST_ERROR: Final = 2003
CR_IPSOCK_ERROR: Final = 2004
CR_UNKNOWN_HOST: Final = 2005
CR_SERVER_GONE_ERROR: Final = 2006
CR_VERSION_ERROR: Final = 2007
CR_OUT_OF_MEMORY: Final = 2008
CR_WRONG_HOST_INFO: Final = 2009
CR_LOCALHOST_CONNECTION: Final = 2010
CR_TCP_CONNECTION: Final = 2011
CR_SERVER_HANDSHAKE_ERR: Final = 2012
CR_SERVER_LOST: Final = 2013
CR_COMMANDS_OUT_OF_SYNC: Final = 2014
CR_NAMEDPIPE_CONNECTION: Final = 2015
CR_NAMEDPIPEWAIT_ERROR: Final = 2016
CR_NAMEDPIPEOPEN_ERROR: Final = 2017
CR_NAMEDPIPESETSTATE_ERROR: Final = 2018
CR_CANT_READ_CHARSET: Final = 2019
CR_NET_PACKET_TOO_LARGE: Final = 2020
CR_EMBEDDED_CONNECTION: Final = 2021
CR_PROBE_SLAVE_STATUS: Final = 2022
CR_PROBE_SLAVE_HOSTS: Final = 2023
CR_PROBE_SLAVE_CONNECT: Final = 2024
CR_PROBE_MASTER_CONNECT: Final = 2025
CR_SSL_CONNECTION_ERROR: Final = 2026
CR_MALFORMED_PACKET: Final = 2027
CR_WRONG_LICENSE: Final = 2028
CR_NULL_POINTER: Final = 2029
CR_NO_PREPARE_STMT: Final = 2030
CR_PARAMS_NOT_BOUND: Final = 2031
CR_DATA_TRUNCATED: Final = 2032
CR_NO_PARAMETERS_EXISTS: Final = 2033
CR_INVALID_PARAMETER_NO: Final = 2034
CR_INVALID_BUFFER_USE: Final = 2035
CR_UNSUPPORTED_PARAM_TYPE: Final = 2036
CR_SHARED_MEMORY_CONNECTION: Final = 2037
CR_SHARED_MEMORY_CONNECT_REQUEST_ERROR: Final = 2038
CR_SHARED_MEMORY_CONNECT_ANSWER_ERROR: Final = 2039
CR_SHARED_MEMORY_CONNECT_FILE_MAP_ERROR: Final = 2040
CR_SHARED_MEMORY_CONNECT_MAP_ERROR: Final = 2041
CR_SHARED_MEMORY_FILE_MAP_ERROR: Final = 2042
CR_SHARED_MEMORY_MAP_ERROR: Final = 2043
CR_SHARED_MEMORY_EVENT_ERROR: Final = 2044
CR_SHARED_MEMORY_CONNECT_ABANDONED_ERROR: Final = 2045
CR_SHARED_MEMORY_CONNECT_SET_ERROR: Final = 2046
CR_CONN_UNKNOW_PROTOCOL: Final = 2047
CR_INVALID_CONN_HANDLE: Final = 2048
CR_SECURE_AUTH: Final = 2049
CR_FETCH_CANCELED: Final = 2050
CR_NO_DATA: Final = 2051
CR_NO_STMT_METADATA: Final = 2052
CR_NO_RESULT_SET: Final = 2053
CR_NOT_IMPLEMENTED: Final = 2054
CR_SERVER_LOST_EXTENDED: Final = 2055
CR_STMT_CLOSED: Final = 2056
CR_NEW_STMT_METADATA: Final = 2057
CR_ALREADY_CONNECTED: Final = 2058
CR_AUTH_PLUGIN_CANNOT_LOAD: Final = 2059
CR_DUPLICATE_CONNECTION_ATTR: Final = 2060
CR_AUTH_PLUGIN_ERR: Final = 2061
CR_INSECURE_API_ERR: Final = 2062
CR_FILE_NAME_TOO_LONG: Final = 2063
CR_SSL_FIPS_MODE_ERR: Final = 2064
CR_DEPRECATED_COMPRESSION_NOT_SUPPORTED: Final = 2065
CR_COMPRESSION_WRONGLY_CONFIGURED: Final = 2066
CR_KERBEROS_USER_NOT_FOUND: Final = 2067
CR_LOAD_DATA_LOCAL_INFILE_REJECTED: Final = 2068
CR_LOAD_DATA_LOCAL_INFILE_REALPATH_FAIL: Final = 2069
CR_DNS_SRV_LOOKUP_FAILED: Final = 2070
CR_MANDATORY_TRACKER_NOT_FOUND: Final = 2071
CR_INVALID_FACTOR_NO: Final = 2072
CR_ERROR_LAST: Final = 2072



from datetime import date, datetime, time, timedelta

Date: type[date]
Time: type[time]
TimeDelta: type[timedelta]
Timestamp: type[datetime]

def DateFromTicks(ticks: float | None) -> date: ...
def TimeFromTicks(ticks: float | None) -> time: ...
def TimestampFromTicks(ticks: float | None) -> datetime: ...



from typing import Final

COM_SLEEP: Final[int]
COM_QUIT: Final[int]
COM_INIT_DB: Final[int]
COM_QUERY: Final[int]
COM_FIELD_LIST: Final[int]
COM_CREATE_DB: Final[int]
COM_DROP_DB: Final[int]
COM_REFRESH: Final[int]
COM_SHUTDOWN: Final[int]
COM_STATISTICS: Final[int]
COM_PROCESS_INFO: Final[int]
COM_CONNECT: Final[int]
COM_PROCESS_KILL: Final[int]
COM_DEBUG: Final[int]
COM_PING: Final[int]
COM_TIME: Final[int]
COM_DELAYED_INSERT: Final[int]
COM_CHANGE_USER: Final[int]
COM_BINLOG_DUMP: Final[int]
COM_TABLE_DUMP: Final[int]
COM_CONNECT_OUT: Final[int]
COM_REGISTER_SLAVE: Final[int]
COM_STMT_PREPARE: Final[int]
COM_STMT_EXECUTE: Final[int]
COM_STMT_SEND_LONG_DATA: Final[int]
COM_STMT_CLOSE: Final[int]
COM_STMT_RESET: Final[int]
COM_SET_OPTION: Final[int]
COM_STMT_FETCH: Final[int]
COM_DAEMON: Final[int]
COM_BINLOG_DUMP_GTID: Final[int]
COM_END: Final[int]



from typing import Final

NOT_NULL: Final = 1
PRI_KEY: Final = 2
UNIQUE_KEY: Final = 4
MULTIPLE_KEY: Final = 8
BLOB: Final = 16
UNSIGNED: Final = 32
ZEROFILL: Final = 64
BINARY: Final = 128
ENUM: Final = 256
AUTO_INCREMENT: Final = 512
TIMESTAMP: Final = 1024
SET: Final = 2048
PART_KEY: Final = 16384
GROUP: Final = 32767
UNIQUE: Final = 65536
